require_dependency "asset_management/asset_management_controller"

module AssetManagement
  class ViewController < AssetManagementController
    def original
      # display original in all it's glory
      if params[:filepath].blank?
        render text: 'File not found!'
      else

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        if /^image/.match?(asset_data.mime_type)
          begin
            image_data = load_magick_image(asset_data)

            send_data image_data.to_blob,
                      filename: asset_data.filename, type: asset_data.mime_type,
                      size: asset_data.size, disposition: "inline"
          rescue MiniMagick::Error
            logger.error "ERROR: #{$!}\n"
            logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
            # TODO: remove - RMagick
            # rescue Magick::ImageMagickError
            #   logger.error "ERROR: " + $! + "\n"
            #   logger.error "#{$0}: ImageMagickError - #{$!}"
          end
        else
          send_data asset_data.data,
                    filename: asset_data.filename, type: asset_data.mime_type,
                    size: asset_data.size, disposition: "inline"
        end
      end
    end

    def thumbnail
      # asset_data.thumbnail(width, height)
      if params[:filepath].blank?
        render text: 'File not found!'
      else

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        if params[:size].nil?
          width  = 75
          height = 75
        else
          width  = params[:size].split("x").first.to_i
          height = params[:size].split("x").last.to_i
        end

        begin
          original_image = load_magick_image(asset_data)
          image_data = original_image.thumbnail "#{width}x#{height}^"

          # TODO: remove - RMagick
          # image_data = original_image.thumbnail(width, height)

          send_data image_data.to_blob,
                    filename: asset_data.filename, type: asset_data.mime_type,
                    size: asset_data.size, disposition: "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
          # TODO: remove - RMagick
          # rescue Magick::ImageMagickError
          #   logger.error "ERROR: " + $! + "\n"
          #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      end
    end

    def flickr_thumbnail
      # asset_data.resize_to_fill(width, height)
      if params[:filepath].blank?
        render text: 'File not found!'
      else

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        if params[:size].nil?
          width  = 75
          height = 75
        else
          width  = params[:size].split("x").first.to_i
          height = params[:size].split("x").last.to_i
        end

        begin
          original_image = load_magick_image(asset_data)
          image_data = original_image.resize "#{width}x#{height}!"

          # TODO: remove - RMagick
          # image_data = original_image.resize_to_fill(width, height)

          send_data image_data.to_blob,
                    filename: asset_data.filename, type: asset_data.mime_type,
                    size: asset_data.size, disposition: "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
          # TODO: remove - RMagick
          # rescue Magick::ImageMagickError
          #   logger.error "ERROR: " + $! + "\n"
          #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      end
    end

    def scaled
      # asset_data.resize_to_fit(width, height)
      if params[:filepath].blank?
        render text: 'File not found!'
      else

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        begin
          original_image = load_magick_image(asset_data)

          if params[:size].nil?
            image_data = original_image
          else
            width  = params[:size].split("x").first.to_i
            height = params[:size].split("x").last.to_i
            image_data = original_image.resize "#{width}x#{height}"

            # TODO: remove - RMagick
            # image_data = original_image.resize_to_fit(width, height)
          end

          send_data image_data.to_blob,
                    filename: asset_data.filename, type: asset_data.mime_type,
                    size: asset_data.size, disposition: "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
          # TODO: remove - RMagick
          # rescue Magick::ImageMagickError
          #   logger.error "ERROR: " + $! + "\n"
          #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      end
    end

    def scale
      # asset_data.resize(scale_factor)
      if params[:filepath].blank?
        render text: 'File not found!'
      else

        asset_data = find_asset_data_from_file_path(
          params[:format].blank? ? params[:filepath] : params[:filepath] + '.' + params[:format]
        )

        begin
          original_image = load_magick_image(asset_data)

          if params[:magnification].nil?
            image_data = original_image
          else
            scale_factor = params[:magnification].to_f / 100.0
            image_data = original_image.resize "#{scale_factor * 100}%"

            # TODO: remove - RMagick
            # image_data = original_image.resize(scale_factor)
          end

          send_data image_data.to_blob,
                    filename: asset_data.filename, type: asset_data.mime_type,
                    size: asset_data.size, disposition: "inline"
        rescue MiniMagick::Error
          logger.error "ERROR: #{$!}\n"
          logger.error "#{$0}: MiniMagick::Error - #{$!}\n"
          # TODO: remove - RMagick
          # rescue Magick::ImageMagickError
          #   logger.error "ERROR: " + $! + "\n"
          #   logger.error "#{$0}: ImageMagickError - #{$!}"
        end
      end
    end

    private

    def find_asset_data_from_file_path(full_path)
      # directory = File.dirname(full_path)
      filename = File.basename(full_path)

      AssetData.find_by(filename: filename)
    end

    def load_magick_image(asset_data)
      asset_data.nil? ? nil : asset_data.load_magick_image
    end
  end
end
