class AssetManagementInitialSchema < ActiveRecord::Migration[4.2]
  def change
    ActiveRecord::Base.transaction do
      create_table :asset_management_virtual_folders do |t|
        t.string :name, null: false
        t.text :description
        t.belongs_to :parent_folder # , null: false

        t.timestamps null: false
      end

      add_index :asset_management_virtual_folders, :parent_folder_id
      add_index :asset_management_virtual_folders, [:name, :parent_folder_id], unique: true,
                                                                               name: :index_virtual_folders_on_name_and_parent_folder_id

      create_table :asset_management_asset_data do |t|
        t.belongs_to :asset_owner, polymorphic: true, null: true

        t.string     :name
        t.text       :description
        t.string     :filename, null: false

        # Trigger the use of a big BLOB in MySQL
        if ActiveRecord::Base.connection.adapter_name == 'MySQL'
          t.binary    :data, limit: 16.megabytes # , null: false
        else
          t.binary    :data # , null: false
        end

        t.integer    :size # , null: false
        t.string     :mime_type # , null: false

        t.belongs_to :folder, null: false

        t.integer    :version, null: false
        t.string     :storage_reference # , null: false

        t.timestamps null: false
      end

      add_index :asset_management_asset_data, :name

      add_index :asset_management_asset_data, [:filename, :folder_id, :version],
                unique: true,
                name: :index_asset_data_on_filename_and_folder_and_version
    end
  end
end
